@@ -1,12 +1,12 @@
 package Beam::Emitter;
 # ABSTRACT: Role for event emitting classes
-$Beam::Emitter::VERSION = '1.000';
+$Beam::Emitter::VERSION = '1.001';
 use strict;
 use warnings;
 
 use Moo::Role;
 use Types::Standard qw(:all);
-use Scalar::Util qw( refaddr );
+use Scalar::Util qw( weaken refaddr );
 use Carp qw( croak );
 use Beam::Event;
 
@@ -22,7 +22,11 @@ has _listeners => (
 sub subscribe {
     my ( $self, $name, $sub ) = @_;
     push @{ $self->_listeners->{$name} }, $sub;
-    return;
+    weaken $self;
+    weaken $sub;
+    return sub {
+        $self->unsubscribe($name => $sub);
+    };
 }
 
 
@@ -84,7 +88,7 @@ Beam::Emitter - Role for event emitting classes
 
 =head1 VERSION
 
-version 1.000
+version 1.001
 
 =head1 SYNOPSIS
 
@@ -251,6 +255,42 @@ Subscribe to an event from this object. C<event_name> is the name of the event.
 C<subref> is a subroutine reference that will get either a L<Beam::Event> object
 (if using the L<emit> method) or something else (if using the L<emit_args> method).
 
+Returns a coderef that, when called, unsubscribes the new subscriber.
+
+    my $unsubscribe = $emitter->subscribe( open_door => sub {
+        warn "ding!";
+    } );
+    $emitter->emit( 'open_door' );  # ding!
+    $unsubscribe->();
+    $emitter->emit( 'open_door' );  # no ding
+
+This unsubscribe subref makes it easier to stop our subscription in a safe,
+non-leaking way:
+
+    my $unsub;
+    $unsub = $emitter->subscribe( open_door => sub {
+        $unsub->(); # Only handle one event
+    } );
+    $emitter->emit( 'open_door' );
+
+The above code does not leak memory, but the following code does:
+
+    # Create a memory cycle which must be broken manually
+    my $cb;
+    $cb = sub {
+        my ( $event ) = @_;
+        $event->emitter->unsubscribe( open_door => $cb ); # Only handle one event
+        # Because the callback sub ($cb) closes over a reference to itself
+        # ($cb), it can never be cleaned up unless something breaks the
+        # cycle explicitly.
+    };
+    $emitter->subscribe( open_door => $cb );
+    $emitter->emit( 'open_door' );
+
+The way to fix this second example is to explicitly C<undef $cb> inside the callback
+sub. Forgetting to do that will result in a leak. The returned unsubscribe coderef
+does not have this issue.
+
 =head2 on ( event_name, subref )
 
 Alias for L</subscribe>.
@@ -296,12 +336,22 @@ Coordinating Christmas Dinner with Beam::Emitter by Yanick Champoux.
 
 Doug Bell <preaction@cpan.org>
 
-=head1 CONTRIBUTOR
+=head1 CONTRIBUTORS
 
-=for stopwords Joel Berger
+=for stopwords Joel Berger Yanick Champoux
+
+=over 4
+
+=item *
 
 Joel Berger <joel.a.berger@gmail.com>
 
+=item *
+
+Yanick Champoux <yanick@babyl.dyndns.org>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2015 by Doug Bell.
